#Script determining the apriori and posteriori errors of the DFT IntML approach in comparison to DFT results. 
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
import numpy as np
import seaborn as sns
import pandas as pd
from matplotlib.lines import Line2D
from matplotlib import pyplot as plt
import matplotlib as mpl
from sklearn.metrics import  mean_absolute_error

#TeX settings
mpl.rcParams.update(mpl.rcParamsDefault)
plt.rcParams['text.usetex'] = True
mpl.rcParams['font.family'] = ['Arial']

#Load Data
df=pd.read_csv("../Data/data.csv").reset_index(drop=True)
dfClean=pd.DataFrame()

#Extract relevant properties from data and determine error
def getPredGivenABatch(batchConv,predictions,batchNo):
    if batchNo==1:
        batchConv=batchConv[batchNo == batchConv['Converged in Batch']]
        batchConv=batchConv[["mA","mB","mC","subst","mmA","mmB","mmC"]]
    else:
        batchConv=batchConv[batchNo == batchConv['Converged in Batch']]
        batchConv=batchConv[["mA","mB","mC","subst","mmA","mmB","mmC","ILD01","ILD12","ILD23"]]
    pdTot=pd.concat([predictions, batchConv], ignore_index=True, sort=False)
    pdTot.reset_index()
    pdTot["converged"]=pdTot[["mA","mB","mC","subst"]].duplicated(keep=False)
    pdTot["converged2"]=pdTot[["mA","mB","mC","subst"]].duplicated(keep="last")
    res=pdTot.query("converged==True").query("converged2==False").sort_values(by=["mA","mB","mC","subst"])
    rConvPred=pdTot.query("converged==True").query("converged2==True").sort_values(by=["mA","mB","mC","subst"])
    if batchNo==1: arr=["mmA","mmB","mmC",]
    else:  arr=["mmA","mmB","mmC","ILD01","ILD12","ILD23"]
    err=np.zeros(0)
    for i in arr:
        err=np.append(err,mean_absolute_error(res[i], rConvPred[i]))
    return err

#Combine dataframes across batches
def makeDF(arra,batch):
    if batch==1:
        dic={"batch": [batch,batch,batch], "lay":["A","B","C"],"Real MM MAE":[arra[0],arra[1],arra[2]]}
    else:
        dic={"batch": [batch,batch,batch], "lay":["A","B","C"],"Real ILD MAE":[arra[3],arra[4],arra[5]],"Real MM MAE":[arra[0],arra[1],arra[2]]}
    return pd.DataFrame.from_dict(dic)

#Load model test metrics and DFT IntML predictions
met=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLMetrics.csv").reset_index(drop=True)
b1=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt1.csv").reset_index(drop=True)
dfReal=makeDF(getPredGivenABatch(df,b1,1), 1)
b2=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt2.csv").reset_index(drop=True)
dfReal= dfReal._append(makeDF(getPredGivenABatch(df,b2,2), 2), ignore_index=True)
b3=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt3.csv").reset_index(drop=True)
dfReal= dfReal._append(makeDF(getPredGivenABatch(df,b3,3), 3), ignore_index=True)
b4=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt4.csv").reset_index(drop=True)
dfReal= dfReal._append(makeDF(getPredGivenABatch(df,b4,4), 4), ignore_index=True)
b5=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt5.csv").reset_index(drop=True)
dfReal= dfReal._append(makeDF(getPredGivenABatch(df,b5,5), 5), ignore_index=True)
met["Real ILD MAE"]=dfReal["Real ILD MAE"]
met["Real MM MAE"]=dfReal["Real MM MAE"]

#Plot posterior error for ILD
sns.set_theme(style="darkgrid")
pal=sns.color_palette(n_colors=3)
sns.lineplot(data=met, x="batch", y="MAE TestILD",hue="lay")
legend_elements=[Line2D([0], [0], color=pal[0], lw=1.5, label='$d_{AB}$'),Line2D([0], [0], color=pal[1], lw=1.5, label='$d_{BC}$'),Line2D([0], [0], color=pal[2], lw=1.5, label='$d_{CSub}$')]
plt.legend(handles=legend_elements,title="Quantity",loc="lower center",ncol=3,bbox_to_anchor=(0.5,1.01))
sns.lineplot(data=met, x="batch", y="Real ILD MAE",hue="lay",linestyle='dashed',legend=False)
plt.xticks([2,3,4,5])
plt.ylabel("Mean Absolute Error [Å]")
plt.xlabel("$n_{\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.tight_layout()
plt.savefig("realWorldPostAnalysisILD.png",dpi=600)
plt.clf()

#Plot posterior error for magnetic moments
sns.set_theme(style="darkgrid")
sns.lineplot(data=met, x="batch", y="Real MM MAE",hue="lay",linestyle='dashed',legend=False)
sns.lineplot(data=met, x="batch", y="MAE Testmm",hue="lay")
plt.legend(title="")
plt.ylabel("Mean Absolute Error [$\\mu_B$]")
plt.xticks([1,2,3,4,5])
legend_elements=[Line2D([0], [0], color=pal[0], lw=1.5, label='$m_{A}$'),Line2D([0], [0], color=pal[1], lw=1.5, label='$m_{B}$'),Line2D([0], [0], color=pal[2], lw=1.5, label='$m_{C}$')]
plt.legend(handles=legend_elements,title="Quantity",loc="lower center",ncol=3,bbox_to_anchor=(0.5,1.01))
plt.xlabel("$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.tight_layout()
plt.savefig("realWorldPostAnalysisMAE.png",dpi=600)
plt.clf()

#Plot apriori test error for ILD
sns.set_theme(style="darkgrid")
sns.lineplot(data=met, x="batch", y="MAE TestILD",hue="lay")
pal=sns.color_palette(n_colors=3)
legend_elements=[Line2D([0], [0], color=pal[0], lw=1.5, label='$d_{AB}$'),Line2D([0], [0], color=pal[1], lw=1.5, label='$d_{BC}$'),Line2D([0], [0], color=pal[2], lw=1.5, label='$d_{CSub}$')]
plt.legend(handles=legend_elements,title="Quantity",loc='center right')
plt.xticks([2,3,4,5])
plt.ylabel("Mean Absolute Error [Å]")
plt.xlabel("$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.tight_layout()
plt.savefig("idealWorldPostAnalysisILD.png",dpi=1200)
plt.clf()

#Plot apriori test error for magnetic moments
sns.set_theme(style="darkgrid")
sns.lineplot(data=met, x="batch", y="MAE Testmm",hue="lay")
plt.ylabel("Mean Absolute Error [$\\mu_B$]")
plt.xticks([1,2,3,4,5])
plt.xlabel("$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
legend_elements=[Line2D([0], [0], color=pal[0], lw=1.5, label='$m_{A}$'),Line2D([0], [0], color=pal[1], lw=1.5, label='$m_{B}$'),Line2D([0], [0], color=pal[2], lw=1.5, label='$m_{C}$')]
plt.legend(handles=legend_elements,title="Quantity",loc='upper center')
plt.tight_layout()
plt.savefig("idealWorldPostAnalysisMAE.png",dpi=1200)
plt.clf()

