#Script compiling visualizations used to discuss if DFT IntML batches are independent of each other and visualized the elemental distributions of inducing and induced magnetic layers.
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
import numpy as np
import seaborn as sns
import pandas as pd
import matplotlib.ticker as mtick 
from matplotlib import pyplot as plt
import matplotlib as mpl
import classMagn

#TeX configs
mpl.rcParams.update(mpl.rcParamsDefault)
plt.rcParams['text.usetex'] = True
mpl.rcParams['font.family'] = ['Arial']

#Load data
df=pd.read_csv("../Data/data.csv").reset_index(drop=True)
dfClean=pd.DataFrame()

#Retrieve ML predicted inputs for each batch
def getPredGivenABatch(batchConv,predictions,batchNo):
    if batchNo==1:
        batchConv=batchConv[batchNo == batchConv['Converged in Batch']]
        print(batchConv.count)
        batchConv=batchConv[["mA","mB","mC","subst","mmA","mmB","mmC"]]
    else:
        batchConv=batchConv[batchNo == batchConv['Converged in Batch']]
        batchConv=batchConv[["mA","mB","mC","subst","mmA","mmB","mmC","ILD01","ILD12","ILD23"]]
    pdTot=pd.concat([predictions, batchConv], ignore_index=True, sort=False).copy()
    pdTot.reset_index()
    pdTot["converged"]=pdTot[["mA","mB","mC","subst"]].duplicated(keep=False)
    pdTot["converged2"]=pdTot[["mA","mB","mC","subst"]].duplicated(keep="last")
    res=pdTot.query("converged==True").query("converged2==False").sort_values(by=["mA","mB","mC","subst"])
    rConvPred=pdTot.query("converged==True").query("converged2==True").sort_values(by=["mA","mB","mC","subst"])
    rConvPred["Converged in Batch"]=batchNo
    return rConvPred

#Retrieve DFT results for each batch
def getDFTGivenABatch(batchConv,predictions,batchNo):
    if batchNo==1:
        batchConv=batchConv[batchNo == batchConv['Converged in Batch']]
        print(batchConv.count)
        batchConv=batchConv[["mA","mB","mC","subst","mmA","mmB","mmC"]]
    else:
        batchConv=batchConv[batchNo == batchConv['Converged in Batch']]
        batchConv=batchConv[["mA","mB","mC","subst","mmA","mmB","mmC","ILD01","ILD12","ILD23"]]
    pdTot=pd.concat([predictions, batchConv], ignore_index=True, sort=False).copy()
    pdTot.reset_index()
    pdTot["converged"]=pdTot[["mA","mB","mC","subst"]].duplicated(keep=False)
    pdTot["converged2"]=pdTot[["mA","mB","mC","subst"]].duplicated(keep="first")
    res=pdTot.query("converged==True").query("converged2==False").sort_values(by=["mA","mB","mC","subst"])
    rConvPred=pdTot.query("converged==True").query("converged2==True").sort_values(by=["mA","mB","mC","subst"])
    rConvPred["Converged in Batch"]=batchNo
    return rConvPred

#Retrieve DFT results and ML predictions for each batch
met=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLMetrics.csv").reset_index(drop=True)
b1=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt1.csv").reset_index(drop=True)
dfReal=getPredGivenABatch(df,b1,1)
b2=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt2.csv").reset_index(drop=True)
dfReal= dfReal._append(getPredGivenABatch(df,b2,2), ignore_index=True)
b3=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt3.csv").reset_index(drop=True)
dfReal= dfReal._append(getPredGivenABatch(df,b3,3), ignore_index=True)
b4=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt4.csv").reset_index(drop=True)
dfReal= dfReal._append(getPredGivenABatch(df,b4,4), ignore_index=True)
b5=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt5.csv").reset_index(drop=True)
dfReal= dfReal._append(getPredGivenABatch(df,b5,5), ignore_index=True)
met=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLMetrics.csv").reset_index(drop=True)
b1=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt1.csv").reset_index(drop=True)
dfDFT=getDFTGivenABatch(df,b1,1)
b2=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt2.csv").reset_index(drop=True)
dfDFT= dfDFT._append(getDFTGivenABatch(df,b2,2), ignore_index=True)
b3=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt3.csv").reset_index(drop=True)
dfDFT= dfDFT._append(getDFTGivenABatch(df,b3,3), ignore_index=True)
b4=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt4.csv").reset_index(drop=True)
dfDFT= dfDFT._append(getDFTGivenABatch(df,b4,4), ignore_index=True)
b5=pd.read_csv("../DFTIntMLPredictions/Predictions/DFTIntMLIt5.csv").reset_index(drop=True)
dfDFT= dfDFT._append(getDFTGivenABatch(df,b5,5), ignore_index=True)
dfDftLat=dfDFT.copy()
dfRealLat=dfReal.copy()

#Computation & visualization of deviation from mean in each batch
aggreg=dfReal.groupby(["Converged in Batch"]).agg(np.mean)
lables=["mmA","mmB","mmC","ILD01","ILD12","ILD23"]
for i in lables:
	sns.barplot(data=aggreg, x=np.arange(1,6,1), y=i)
	sns.set_theme(style="whitegrid")
	plt.xlabel("Converged in Batch")
	plt.ylabel(i+" abs change to mean of all predictions")
	plt.tight_layout()
	plt.savefig(i+"",dpi=600)#Guess- Res =Disp
	plt.clf()

#Compute actual "displacements" for ML and DFT results
dfuned=pd.read_csv("../Data/data.csv").reset_index(drop=True).query("`Converged in Batch`==0")[["mmA","mmB","mmC","ILD01","ILD12","ILD23","Converged in Batch"]]
dfuned["$\\Delta m_A$"]=-1+dfuned["mmA"]
dfuned["$\\Delta m_B$"]=-1+dfuned["mmB"]
dfuned["$\\Delta m_C$"]=-1+dfuned["mmC"]
dfuned["Converged in Batch"]=0
dfDFT["$\\Delta m_A$"]=-(dfReal["mmA"]-dfDFT["mmA"])
dfDFT["$\\Delta m_B$"]=-(dfReal["mmB"]-dfDFT["mmB"])
dfDFT["$\\Delta m_C$"]=-(dfReal["mmC"]-dfDFT["mmC"])
dfDFT=pd.concat([dfDFT.query("`Converged in Batch`>0"),dfuned],ignore_index=True).copy()
dfDFT["Converged in Batch"]=dfDFT["Converged in Batch"].astype(int)

#Visualize magnetic displacement
sns.set_theme(style="whitegrid")
dataMelt=pd.melt(dfDFT,id_vars=["Converged in Batch"],value_name="Magnetic Difference",var_name="Layer Displacement",value_vars=["$\\Delta m_A$","$\\Delta m_B$","$\\Delta m_C$"])
sns.boxplot(data=dataMelt,x="Magnetic Difference",y="Layer Displacement",hue="Converged in Batch",meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"3"},showmeans=True)
plt.ylabel("")
plt.xlabel("$\\Delta m$ in $\\mu_B$")
plt.legend(title="$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$",loc='lower center', bbox_to_anchor=(0.5, 1.05),
          fancybox=True, ncol=6)
plt.tight_layout()
plt.savefig("displMMBox.png",dpi=1200)
plt.clf()

#Load data
df1 = pd.DataFrame()
df=pd.read_csv("../Data/data.csv").reset_index(drop=True)
#Compute couplings for induced moments in magnetic layers
for i in range (0,df.shape[0]):
	arr=classMagn.couplingNoMag(df[["mmA","mmB","mmC"]].iloc[i].values[0],df[["mmA","mmB","mmC"]].iloc[i].values[1],df[["mmA","mmB","mmC"]].iloc[i].values[2])

	if arr is not None:
		su=""
		for j in [arr[2],arr[3],arr[4]]:
			if j is None: continue
			su=su+j
		if "A" in su: a="A"
		else: a="No"
		if "B" in su: b="B"
		else: b="No"
		if "C" in su: c="C"
		else: c="No"
		df2 = pd.DataFrame({"mA":df["mA"].iloc[i] ,"mB":df["mB"].iloc[i],"mC":df["mC"].iloc[i], "Ind Site A": a,"Ind Site B": b,"Ind Site C": c, "ABInt":arr[0] , "BCInt": arr[1],"MagnNeighA":arr[5],"MagnNeighB":arr[6]},index=[i])
		df1=pd.concat([df1, df2])

#Visualize induced site relations
df1=pd.melt(df1,id_vars=["mA","mB","mC","ABInt","BCInt","MagnNeighA","MagnNeighB"], value_vars=["Ind Site A","Ind Site B","Ind Site C"], var_name=["Induced Site"]).query("value!='No'").query("mA!=0").query("mB!=0")
df2=pd.DataFrame()
df2[["Atomic Number","Interaction Type"]]=df1.query("value=='C'")[['mC','BCInt']]
df3=pd.DataFrame()
df3[["Atomic Number","Interaction Type"]]=df1.query("value=='B'")[['mB','BCInt']]
df3["Interaction Type"]=0.5*(df3["Interaction Type"]+df1.query("value=='B'")["ABInt"])
df4=pd.DataFrame()
df4[["Atomic Number","Interaction Type"]]=df1.query("value=='A'")[['mA','ABInt']]
df3=pd.concat([df4,df3])
df3=pd.concat([df2,df3])
df3["Atomic Number"]=df3["Atomic Number"].astype("category")
df3["Atomic Number"]=df3["Atomic Number"].astype("category")
df3Neg=df3.query("`Interaction Type`<0")
lena=len(df3Neg)
df3NegCount=df3Neg.groupby("Atomic Number").agg('count')
df3NegCount["Relative Count"]=df3NegCount["Interaction Type"]
df3NegCount["Interaction Type"]="AFM"
df3pos=df3.query("`Interaction Type`>0")
lenb=len(df3pos)
df3posCount=df3pos.groupby("Atomic Number").agg('count')
df3posCount["Relative Count"]=df3posCount["Interaction Type"]
df3posCount["Interaction Type"]="FM"
df3pos=pd.concat([df3posCount,df3NegCount])
df3pos=df3pos.add_suffix('').reset_index()
df3pos["Atomic Number"]=df3pos["Atomic Number"].astype("category")
df3pos["Relative Count"]=df3pos["Relative Count"]/(lena+lenb)
fig,ax=plt.subplots()
sns.barplot(data=df3pos,y="Relative Count",x="Atomic Number",hue="Interaction Type",palette=sns.color_palette("colorblind",2))
plt.xlim(-0.5,9.5)
ax.yaxis.set_major_formatter(mtick.PercentFormatter(1.0))
plt.xlabel("Element at Induced Site")
plt.xticks(ticks=[0,1,2,3,4,5,6,7,8,9],labels=["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"])
plt.ylabel("Relative Count of Computed Magnetic Pair Interactions")
plt.tight_layout()
plt.savefig("bar.png",dpi=1200)
plt.clf()

#Visualize inducing site relations
df1["valOld"]=df1["value"]
df1=pd.melt(df1[["mA","mB","mC","ABInt","BCInt","MagnNeighA","MagnNeighB","valOld"]],id_vars=["mA","mB","mC","ABInt","BCInt","valOld"], value_vars=["MagnNeighA","MagnNeighB"], var_name=["Inducing Site"]).query("value!='No'").query("mA!=0").query("mB!=0")
df2=pd.DataFrame()
df2[["Atomic Number inducing Site","Interaction Type"]]=df1.query("value=='C'")[['mC','BCInt']]
df3=pd.DataFrame()
df3[["Atomic Number inducing Site","Interaction Type"]]=df1.query("value=='B'")[['mB','BCInt']]
df3["Interaction Type"]=0.5*(df3["Interaction Type"]+df1.query("value=='B'")["ABInt"])
df4=pd.DataFrame()
df4[["Atomic Number inducing Site","Interaction Type"]]=df1.query("value=='A'")[['mA','ABInt']]
df3=pd.concat([df4,df3])
df3=pd.concat([df2,df3])
df3["Atomic Number inducing Site"]=df3["Atomic Number inducing Site"].astype("category")
df3Neg=df3.query("`Interaction Type`<0")
lena=len(df3Neg)
df3NegCount=df3Neg.groupby("Atomic Number inducing Site").agg('count')
df3NegCount["Relative Count"]=df3NegCount["Interaction Type"]
df3NegCount["Interaction Type"]="AFM"
df3pos=df3.query("`Interaction Type`>0")
lenb=len(df3pos)
df3posCount=df3pos.groupby("Atomic Number inducing Site").agg('count')
df3posCount["Relative Count"]=df3posCount["Interaction Type"]
df3posCount["Interaction Type"]="FM"
df3pos=pd.concat([df3posCount,df3NegCount])
df3pos=df3pos.add_suffix('').reset_index()
df3pos["Atomic Number inducing Site"]=df3pos["Atomic Number inducing Site"].astype("category")
df3pos["Relative Count"]=df3pos["Relative Count"]/(lena+lenb)
fig,ax=plt.subplots()
sns.barplot(data=df3pos,y="Relative Count",x="Atomic Number inducing Site",hue="Interaction Type",palette=sns.color_palette("colorblind",2))
plt.xlabel("Element at Inducing Site")
plt.xticks(ticks=[0,1,2,3,4,5,6,7,8,9],labels=["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"])
plt.xlim(-0.5,9.5)
ax.yaxis.set_major_formatter(mtick.PercentFormatter(1.0))
plt.ylabel("Relative Count of Computed Magnetic Pair Interactions")
plt.tight_layout()
plt.savefig("barind.png",dpi=1200)

