#Script which was used to predict the input parameters for future DFT IntML iterations
#This also includes some XAI (SHAP) discussion of feature impacts
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import shap
from sklearn.preprocessing import StandardScaler
from sklearn.model_selection import train_test_split
#Model
from xgboost import XGBRegressor
#Utils/stat tools and XAI
import mlUtils
import shap

#Define radom seed
randomSeed=31415

#I/o load data
filename="../Data/data.csv"
headers=np.genfromtxt(filename,delimiter=",",dtype=str)[0,:]
data=np.genfromtxt(filename,delimiter=",")[1:,:]
dfPand=pd.read_csv("../Data/data.csv")
#Split x,y
x=data[:,0:4]#dfPand["mA","mB","mC","subst"]
y=data[:,[8,9,10,4,5,6]]#dfPand["mma","mmb","mmc","ILD01","ILD12","ILD23"]
#Split and shuffle data to later obtain test scores
trainX,testX,trainY,testY=train_test_split(x,y,test_size=0.2, shuffle=True ,random_state=randomSeed)

#Scaling
scaler=StandardScaler()
scaler.fit(trainX)
trainX=scaler.transform(trainX)
testX=scaler.transform(testX)
x=scaler.transform(x)

#Load not relaxed films composition
remDataOr=pd.read_csv("DFTIntMLPredictions/Predictions/nonConvWithoutMLPred.csv")[["mA","mB","mC","subst"]]
remData=scaler.transform(remDataOr)

#Function which does the HPO, chooses model, evaluated and refits on entire data before prediction
def fitEvalRefitFullSetPred(trainX,trainY,testX,testY,x,y,remData):
    xgBoost=XGBRegressor()
    xgBoostSet={'n_estimators': [4,10,100,1000,5000],'eta':[0.01,0.05,0.1,0.2,0.3],"lambda":[1.1,1.05,1]}
    #xgBoostSet={'n_estimators': [10]}
    model=mlUtils.opt(xgBoost,'neg_mean_absolute_error',trainX,trainY,xgBoostSet)
    mlUtils.modelEvalReg(model,"xgBoost",testY,trainY,testX,trainX,x,y,'neg_mean_absolute_error')
    model.fit(x,y)
    prediction=model.predict(remData)
    return prediction,model

#Call routine
predicts=np.zeros((len(remData[:,0]),6))

#SHAP discussion
dfSHAP=pd.DataFrame(columns=["Feature","Prediction","Value","Error"])
lab=["$m_A$","$m_B$","$m_C$","$d_{AB}$","$d_{BC}$","$d_{CSub}$"]
rowOrder=["$Z_A$","$Z_B$","$Z_C$","$Z_{Sub}$"]
for i in range (0,6):
    predicts[:,i],model=fitEvalRefitFullSetPred(trainX,trainY[:,i],testX,testY[:,i],x,y[:,i],remData)
    explainer = shap.TreeExplainer(model)
    shap_values = explainer.shap_values(remData)
    #Define neighboring degree
    if i in [0]:neigh=["0 Order","1 Order","2 Order","3 Order"]
    if i in [3]: neigh=["0 Order","0 Order","1 Order","2 Order"]
    if i in [1]:neigh=["1 Order","0 Order","1 Order","2 Order"]
    if i in [4]: neigh=["1 Order","0 Order","0 Order","1 Order"]
    if i in [2]:neigh=["2 Order","1 Order","0 Order","1 Order"]
    if i in [5]: neigh=["2 Order","1 Order","0 Order","0 Order"]
    new_row={"Feature": "$Z_A$","Value": np.mean(np.abs(shap_values),axis=0)[0],"Prediction": lab[i],"Neighbour Degree": neigh[0],"Error":np.std(np.abs(shap_values),axis=0)[0]
   }
    dfSHAP=pd.concat([dfSHAP, pd.DataFrame([new_row])], ignore_index=True)
    new_row={"Feature": "$Z_B$","Value": np.mean(np.abs(shap_values),axis=0)[1],"Prediction": lab[i],"Neighbour Degree": neigh[1],"Error":np.std(np.abs(shap_values),axis=0)[1]
   }
    dfSHAP=pd.concat([dfSHAP, pd.DataFrame([new_row])], ignore_index=True)
    new_row={"Feature": "$Z_C$","Value": np.mean(np.abs(shap_values),axis=0)[2],"Prediction": lab[i],"Neighbour Degree": neigh[2],"Error":np.std(np.abs(shap_values),axis=0)[2]
   }
    dfSHAP=pd.concat([dfSHAP, pd.DataFrame([new_row])], ignore_index=True)
    new_row={"Feature": "$Z_{Sub}$","Value": np.mean(np.abs(shap_values),axis=0)[3],"Prediction": lab[i],"Neighbour Degree": neigh[3],"Error":np.std(np.abs(shap_values),axis=0)[3]
   }
    dfSHAP=pd.concat([dfSHAP, pd.DataFrame([new_row])], ignore_index=True)

#Average SHAP value magnitudes per feature
sns.set_theme(style="whitegrid")
sns.lineplot(data=dfSHAP,x="Feature",y="Value",hue="Prediction")
plt.ylabel("Average Magnitude of SHAP Values")
plt.legend(title="Quantity",frameon="False")
plt.xlabel("Model Input Feature")
plt.tight_layout()
plt.savefig("shapPred.png",dpi=1200)
plt.clf()

#Impact of atomic numbers on ILD
sns.set_theme(style="whitegrid")
sns.scatterplot(data=dfSHAP.query("`Prediction`=='$d_{AB}$' | `Prediction`=='$d_{BC}$' | `Prediction`=='$d_{CSub}$'"),x="Neighbour Degree",y="Value",hue="Prediction",palette=sns.color_palette("rocket",3),s=50)
plt.ylabel("Average Magnitude of SHAP Values")
plt.xlabel("Neighboring Degree")
plt.legend(title="Quantity")
plt.savefig("shapPredDistNeigh.png",dpi=1200)
plt.clf()

#Impact of atomic numbers on magnetic moment
sns.set_theme(style="whitegrid")
sns.scatterplot(data=dfSHAP.query("`Prediction`=='$m_A$' | `Prediction`=='$m_B$' | `Prediction`=='$m_C$'"),x="Neighbour Degree",y="Value",hue="Prediction",palette=sns.color_palette("rocket",3),s=50)
plt.xlabel("Neighboring Degree")
plt.legend(title="Quantity")
plt.ylabel("Average Magnitude of SHAP Values")
plt.savefig("shapPredMagNeigh.png",dpi=1200)
plt.clf()

pdTot=remDataOr
pdTot["mmA"]=predicts[:,0].tolist()
pdTot["mmB"]=predicts[:,1].tolist()
pdTot["mmC"]=predicts[:,2].tolist()
pdTot["ILD01"]=predicts[:,3].tolist()
pdTot["ILD12"]=predicts[:,4].tolist()
pdTot["ILD23"]=predicts[:,5].tolist()

pdTot.to_csv("Predictions.csv")







