#Script which has been used to get an overview about correlations and distributions of the data.
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
import seaborn as sns
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

#Load Data
df=pd.read_csv("../Data/data.csv").reset_index(drop=True)
corr=df.corr()

#Correlation matrix
up=corr.where(np.triu(np.ones(corr.shape),k=1).astype(bool))
mask = np.triu(np.ones_like(corr, dtype=bool))
cmap = sns.color_palette("vlag", as_cmap=True)
sns.heatmap(corr,mask = mask, cmap=cmap)
plt.tight_layout()
plt.savefig("corr.png",dpi=600)
plt.clf()

#Total Magnetic Moment
df["Converged in Batch"]=df["Converged in Batch"].astype("category")
sns.set_theme(style="whitegrid")
sns.boxplot(data=df,x="totCellMom",y="Converged in Batch",meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},showmeans=True,palette=sns.color_palette("colorblind", 6))
plt.xlabel("$M^{\\mathrm{Tot}}_{\\mathrm{Cell}}$ in $\\mu_B$")
plt.ylabel("$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.yticks(ticks=[0,1,2,3,4,5],labels=[0,1,2,3,4,5])
plt.tight_layout()
plt.savefig("boxMMTotPerBatch.png",dpi=1200)
plt.clf()

dfMag=df.copy()

#Overview plots
g=sns.pairplot(
    df[["Converged in Batch","totIt","ForceIt","totCellMom","totalFilmThick"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("netPair.png",dpi=400)
plt.clf()

g=sns.pairplot(
    df[["ILD01","ILD12","ILD23","ILD34","disA","disB","disC","totalFilmThick"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("pai7plot.png",dpi=400)
plt.clf()

g=sns.pairplot(
    df[["mA","mB","mC","subst","disA","disB","disC"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("pai2plot.png",dpi=400)
plt.clf()

g=sns.pairplot(
    df[["mA","mB","mC","subst","mmA","mmB","mmC","Converged in Batch","totIt","ForceIt"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("paiItMagnplot.png",dpi=400)
plt.clf()

g=sns.pairplot(
    df[["ILD01","ILD12","ILD23","ILD34","disA","disB","disC","Converged in Batch","totIt","ForceIt"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("paiItILDplot.png",dpi=400)
plt.clf()

g=sns.pairplot(
    df[["mmA","mmB","mmC","totCellMom","mmSubA","mmSubB","mmSubC","totalFilmThick"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("pai3plot.png",dpi=400)
plt.clf()

g=sns.pairplot(
    df[["ILD01","ILD12","ILD23","ILD34","mmA","mmB","mmC"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("pai4plot.png",dpi=400)
plt.clf()

g=sns.pairplot(
    df[["mA","mB","mC","subst","ILD01","ILD12","ILD23","ILD34","totalFilmThick"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("pai5plot.png",dpi=400)
plt.clf()

g=sns.pairplot(
    df[["mA","mB","mC","subst","mmA","mmB","mmC","totCellMom"]],
    plot_kws=dict(marker="+", linewidth=1),
    diag_kws=dict(fill=False),
)
g.map_lower(sns.kdeplot, levels=4, color=".2")
g.add_legend()
plt.savefig("pai6plot.png",dpi=400)
plt.clf()

#"Make" all possible elemental conbinations to determine which films did not converge
dfConv=df[["mA","mB","mC","subst"]]
substs=pd.unique(dfConv["subst"])
pdTot=pd.DataFrame(columns=["mA","mB","mC","subst"])
for i in pd.unique(dfConv["mA"]):
    for j in pd.unique(dfConv["mB"]):
        for k in pd.unique(dfConv["mC"]):
            for l in substs:
                if (i==0 and j==0) or (i==0 and j!=0) or (i!=0 and j!=0):
                    pdcon=pd.DataFrame([{"mA":i,"mB":j,"mC":k,"subst":l}])
                    pdTot=pd.concat([pdTot, pdcon], ignore_index=True, sort=False)
pdTot=pd.concat([pdTot, dfConv], ignore_index=True, sort=False)
pdTot["converged"]=pdTot.duplicated(keep="first")

#Visualization of distribution of converged films by elements
sns.set_theme(style="whitegrid")
fig, axs = plt.subplots(2, 2, figsize=(7, 7))
sns.histplot(data=pdTot[(pdTot[['mA']] != 0).all(axis=1)], x="mA", hue="converged",hue_order=[True,False],discrete = True, binwidth=1, ax=axs[0, 0])
sns.histplot(data=pdTot[(pdTot[['mB']] != 0).all(axis=1)], x="mB", hue="converged",hue_order=[True,False],discrete = True,binwidth=1, ax=axs[0, 1])
sns.histplot(data=pdTot, x="mC", hue="converged",hue_order=[True,False],discrete = True, binwidth=1, ax=axs[1, 0])
sns.histplot(data=pdTot, x="subst", hue="converged",hue_order=[True,False],discrete = True, binwidth=1, ax=axs[1, 1])
plt.tight_layout()
plt.savefig('dist.png',dpi=400)
plt.clf()

sns.set_theme(style="whitegrid")
fig, axs = plt.subplots(2, 2, figsize=(7, 7))
sns.histplot(data=df[(df[['mA']] != 0).all(axis=1)], x="mA", hue="Converged in Batch",hue_order = pd.unique(df['Converged in Batch'])[::-1], multiple="stack",discrete = True, binwidth=1, ax=axs[0, 0])
sns.histplot(data=df[(df[['mB']] != 0).all(axis=1)], x="mB", hue="Converged in Batch",hue_order = pd.unique(df['Converged in Batch'])[::-1], multiple="stack",discrete = True,binwidth=1, ax=axs[0, 1])
sns.histplot(data=df, x="mC", hue="Converged in Batch",hue_order = pd.unique(df['Converged in Batch'])[::-1], multiple="stack",discrete = True, binwidth=1, ax=axs[1, 0])
sns.histplot(data=df, x="subst", hue="Converged in Batch",hue_order = pd.unique(df['Converged in Batch'])[::-1], multiple="stack",discrete = True, binwidth=1, ax=axs[1, 1])
plt.tight_layout()
plt.savefig('batchDist.png',dpi=400)
plt.clf()

