#Script compiling extensive plots visualizing magnetic properties as well as the capabilities of DFT-integrated ML
#- Robin Hilgers, 2023
#Published on Zenodo
#--------------------------------------------------------------------------------
# Copyright (c) 2023 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
# This file is available as free software under the conditions
# of the MIT license as expressed in the LICENSE file in more detail.
#--------------------------------------------------------------------------------

#Imports
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib as mpl
import seaborn as sns
import matplotlib.patches as mpatches
from matplotlib.ticker import FuncFormatter

#Tex related configs
mpl.rcParams.update(mpl.rcParamsDefault)
plt.rcParams['text.usetex'] = True
mpl.rcParams['font.family'] = ['Arial']

#Load Data
df=pd.read_csv("../Data/dataInp.txt").reset_index(drop=True)
dfClean=pd.DataFrame()

#Relabelling
oldLabels=["mA","mB","mC","subst","ILD01","ILD12","ILD23","ILD34","ILD01In","ILD12In","ILD23In","mmA","mmB","mmC","totCellMom","mmSubA","mmSubB","mmSubC","disA","disB","disC","totalFilmThick","enHart","enFerm","enBandG","enValEl","spin1charge","spin2charge","totCh","numAtTypes","totIt","ForceIt","Converged in Batch","Force it to 10^-3","Force it to 5*10^-3","Force it to 10^-4","Force it to 5*10^-4"]
newCleanLabels=["$Z_A$","$Z_B$","$Z_C$","$Z_{Subst}$","$d^{ILD}_{A-B}$","$d^{ILD}_{B-C}$","$d^{ILD}_{C-Subst}$","$d_{Subst}^{Lattice}$","$d_{AB}$In","$d_{BC}$In","$d_{CD}$In","$m_A$","$m_B$","$m_C$","$M_{tot}$","$m^{Subst}_{Outer}$","$m^{Subst}_{Next Outer}$","$m^{Subst}_{Center}$","$\\Delta d_A$","$\\Delta d_B$","$\\Delta d_C$","$d_{tot}$","$E_{Hart}$","$E_{F}$","$\\Delta E_{Gap}$","$E_{Val}$","$\\uparrow q$","$\\downarrow q$","$q_{tot}$","$n_{types}$","$n_{Iter}^{Tot}$","$n_{Iter}^{Force}$","Converged in Batch","Force Iterations to $10^{-3}$","Force Iterations to  $5\\cdot 10^{-3}$","Force Iterations to  $ 10^{-4}$","Force Iterations to  $5\\cdot 10^{-4}$"]
for i in range (len(oldLabels)):
	dfClean[newCleanLabels[i]]=df[oldLabels[i]]

df=dfClean

#Correlation plot
corr=df.corr()
up=corr.where(np.triu(np.ones(corr.shape),k=1).astype(bool))
mask = np.triu(np.ones_like(corr, dtype=bool))
cmap = sns.color_palette("vlag", as_cmap=True)
sns.heatmap(corr,mask = mask, cmap=cmap)
plt.tight_layout()
plt.savefig("corrPolished.png",dpi=600)
plt.clf()

#Visualize difference of input and output ILDs "ILD Displacement"
dfClean["$\\Delta d_{AB}$"]=dfClean["$d^{ILD}_{A-B}$"]-dfClean["$d_{AB}$In"]
dfClean["$\\Delta d_{BC}$"]=dfClean["$d^{ILD}_{B-C}$"]-dfClean["$d_{BC}$In"]
dfClean["$\\Delta d_{CSub}$"]=dfClean["$d^{ILD}_{C-Subst}$"]-dfClean["$d_{CD}$In"]
dfClean["Converged in Batch"]=dfClean["Converged in Batch"].astype(int)
sns.set_theme(style="whitegrid")
dataMelt=pd.melt(dfClean,id_vars=["Converged in Batch"],value_name="ILD Difference",var_name="Layer Difference",value_vars=["$\\Delta d_{AB}$","$\\Delta d_{BC}$","$\\Delta d_{CSub}$"])
sns.boxplot(data=dataMelt,x="ILD Difference",y="Layer Difference",hue="Converged in Batch",meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"3"},showmeans=True)
plt.xlabel("$\\Delta d$ in \\AA")
plt.legend(title="$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$",loc='lower center', bbox_to_anchor=(0.5, 1.05),
          fancybox=True, ncol=6)
plt.ylabel("")
plt.tight_layout()
plt.savefig("displBox.png",dpi=1200)
plt.clf()

#Compare induced moments of Substrate A layer and substrate B layer
sns.set_theme(style="whitegrid")
sns.scatterplot(data=dfClean,x="$m^{Subst}_{Outer}$",y="$m^{Subst}_{Next Outer}$")
plt.xlabel("$m_{SubA}$")
plt.ylabel("$m_{SubB}$")
plt.tight_layout()
plt.savefig("mmSubAmmSubB.png",dpi=1200)
plt.clf()

#Compare induced moments of Substrate B layer and substrate C layer
sns.set_theme(style="whitegrid")
sns.scatterplot(data=dfClean,x="$m^{Subst}_{Next Outer}$",y="$m^{Subst}_{Center}$")
plt.xlabel("$m_{SubB}$")
plt.ylabel("$m_{SubC}$")
plt.tight_layout()
plt.savefig("mmSubBmmSubC.png",dpi=1200)
plt.clf()

#Visualization of the convergence rate improval with DFT-integrated ML
sns.set_theme(style="whitegrid")
ax=sns.histplot(data=df,x="Converged in Batch",cumulative=True,stat="count",element="bars",multiple="stack",discrete="True",palette=sns.color_palette("ch:s=.25,rot=-0.25",as_cmap=True))
plt.axhline(y=6660, color="r",linestyle="--")
batchCounts=df["Converged in Batch"].value_counts()
ax.annotate(text="",xytext=(-1.75,0),xy=(-1.75,batchCounts[0.0]),arrowprops={'arrowstyle':"simple",'facecolor':"blue","shrinkA":0,"shrinkB":0},annotation_clip=False)
ax.annotate(text="Initial Guess \n Magn. + Struct.",xytext=(-2.25,batchCounts[0.0]/3),xy=(-2,batchCounts[0.0]/3),annotation_clip=False,rotation=90)
ax.annotate(text="",xytext=(-1.75,batchCounts[0.0]),xy=(-1.75,batchCounts[1.0]+batchCounts[0.0]),arrowprops={'arrowstyle':"simple",'facecolor':"green","shrinkA":0,"shrinkB":0},annotation_clip=False)
ax.annotate(text="Magn. Mom. \n ML Opt.",xytext=(-2.25,batchCounts[0.0]),xy=(-2,batchCounts[0.0]),annotation_clip=False,rotation=90)
ax.annotate(text="",xytext=(1.25,batchCounts[0.0]+batchCounts[1.0]),xy=(1.25,batchCounts[1.0]+batchCounts[0.0]+batchCounts[2.0]+batchCounts[3.0]+batchCounts[4.0]+batchCounts[5.0]),arrowprops={'arrowstyle':"simple",'facecolor':"black","shrinkA":0,"shrinkB":0},annotation_clip=False)
ax.annotate(text="Full ML \n Struc. +\n Magn.",xytext=(0.5,batchCounts[1.0]+batchCounts[0.0]+200),xy=(1,batchCounts[1.0]+batchCounts[0.0]+200),annotation_clip=False,rotation=90)
plt.axhline(y=batchCounts[1.0]+batchCounts[0.0]+batchCounts[2.0]+batchCounts[3.0]+batchCounts[4.0]+batchCounts[5.0], color="black",linestyle="--")
plt.axhline(y=batchCounts[0], color="b",linestyle="--")
plt.axhline(y=batchCounts[1]+batchCounts[0], color="g",linestyle="--")
plt.yticks([0,1000,2000,3000,4000,5000,6000,6660])
plt.tight_layout()
plt.ylabel("Successfully Relaxed Structure Count")
plt.xlabel("$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.savefig("cumul.png",dpi=1200)
plt.clf()

#Prepare dataframe in order to visualize elemental distribution of layers in converged films
dfConv=df[["$Z_A$","$Z_B$","$Z_C$","$Z_{Subst}$","Converged in Batch"]]
substs=pd.unique(dfConv["$Z_{Subst}$"])
pdTot=pd.DataFrame(columns=["$Z_A$","$Z_B$","$Z_C$","$Z_{Subst}$","Converged in Batch"])

#"Make" all films to determine such, which did not successfully relax
for i in pd.unique(dfConv["$Z_A$"]):
    for j in pd.unique(dfConv["$Z_B$"]):
        for k in pd.unique(dfConv["$Z_C$"]):
            for l in substs:
                if (i==0 and j==0) or (i==0 and j!=0) or (i!=0 and j!=0):
                    pdcon=pd.DataFrame([{"$Z_A$":i,"$Z_B$":j,"$Z_C$":k,"$Z_{Subst}$":l,"Converged in Batch": "DNC"}])
                    pdTot=pd.concat([pdcon,pdTot], ignore_index=True, sort=False,copy=True)
pdTot=pd.concat([ pdTot,dfConv], ignore_index=True, sort=False,copy=True)
pdTotCo=pdTot.copy()
pdTotCo["dup"]=pdTot.duplicated(keep=False,subset=["$Z_A$","$Z_B$","$Z_C$","$Z_{Subst}$"])
pdTotCo=pdTotCo.query("dup==False")
pdTot["converged"]=pdTot.duplicated(keep="first",subset=["$Z_A$","$Z_B$","$Z_C$","$Z_{Subst}$"])
pdGoodMix=pd.concat([ pdTotCo,pdTot.query("converged==True")], ignore_index=True, sort=False,copy=True)

#Visualization of the elemental distribution of layers of converged relaxed films
sns.set_theme(style="ticks")
fig, axs = plt.subplots(1,3, figsize=(7, 3))
axs[0].set_xticks(np.arange(21,31,1))
axs[1].set_xticks(np.arange(21,31,1))
axs[2].set_xticks(np.arange(21,31,1))
plt.xlim(20,31)
axs[0].tick_params(axis='x', labelsize=8)
axs[1].tick_params(axis='x', labelsize=8)
axs[2].tick_params(axis='x', labelsize=8)
sns.histplot(data=pdTot[(pdTot[["$Z_A$"]] != 0).all(axis=1)], x="$Z_A$", hue="converged",hue_order=[True,False],discrete = True, binwidth=1, ax=axs[0],legend=False)
sns.histplot(data=pdTot[(pdTot[["$Z_B$"]] != 0).all(axis=1)], x="$Z_B$", hue="converged",hue_order=[True,False],discrete = True,binwidth=1, ax=axs[1],legend=False)
sns.histplot(data=pdTot, x="$Z_C$", hue="converged",hue_order=[True,False],discrete = True, binwidth=1, ax=axs[ 2],legend=False)
blue = mpatches.Patch(color=sns.color_palette()[0]
, label='True')
orange = mpatches.Patch(color=sns.color_palette()[1], label='False')
s = fig.subplotpars
bb=[s.left+0.03, s.top-0.05, s.right-s.left, 0.05 ]
fig.legend([blue,orange],  ["Yes","No"],title="Converged", loc=8, bbox_to_anchor=bb, ncol= 4, borderaxespad=0,
                bbox_transform=fig.transFigure, fancybox=False, edgecolor="k")
plt.tight_layout(rect=[0, 0.03, 1, .88])
plt.savefig('distPolishedMagn.png',dpi=600)
plt.clf()

#Substrate distribution of converged films
fig,(ax,ax2) = plt.subplots(1,2, sharey=True, facecolor='w', gridspec_kw={'width_ratios': [1, 1]},figsize=(5, 3))
sns.histplot(data=pdTot, x="$Z_{Subst}$", hue="converged",hue_order=[True,False],discrete = True, binwidth=1, ax=ax,legend=False)
sns.histplot(data=pdTot, x="$Z_{Subst}$", hue="converged",hue_order=[True,False],discrete = True, binwidth=1, ax=ax2,legend=False)
ax.tick_params(axis='x', labelsize=8)
ax2.tick_params(axis='x', labelsize=8)
ax.set_xticks(np.arange(45,48,1))
ax2.set_xticks(np.arange(77,80,1))
ax.set_xlim(44, 48)
ax2.set_xlim(76, 80)
ax.spines['right'].set_visible(False)
ax2.spines['left'].set_visible(False)
ax.yaxis.tick_left()
ax2.yaxis.tick_right()
d = .015
kwargs = dict(transform=ax.transAxes, color='k', clip_on=False)
ax.plot((1-d,1+d), (-d,+d), **kwargs)
ax.plot((1-d,1+d),(1-d,1+d), **kwargs)
kwargs.update(transform=ax2.transAxes)
ax2.plot((-d,+d), (1-d,1+d), **kwargs)
ax2.plot((-d,+d), (-d,+d), **kwargs)
plt.tight_layout()
plt.savefig("distPolishedSubst.png",dpi=600)

#Visualization of the elemental distribution of layers of converged relaxed films
pdGoodMix["Cono2"]=pdGoodMix.duplicated(keep="first",subset=["$Z_A$","$Z_B$","$Z_C$","$Z_{Subst}$"])
pdGoodMix=pdGoodMix.query("Cono2==False")
pdTot=pdGoodMix
sns.set_theme(style="ticks")
fig, axs = plt.subplots(1,3, figsize=(7, 3))
axs[0].set_xticks(ticks=np.arange(21,31,1),labels=["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"])
axs[1].set_xticks(ticks=np.arange(21,31,1),labels=["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"])
axs[2].set_xticks(ticks=np.arange(21,31,1),labels=["Sc","Ti","V","Cr","Mn","Fe","Co","Ni","Cu","Zn"])
plt.xlim(20,31)
fig.text(0.54, 0.04, 'Elements', ha='center')
axs[0].set_xlabel("Magnetic Layer A")
axs[1].set_xlabel("Magnetic Layer B")
axs[2].set_xlabel("Magnetic Layer C")
axs[0].tick_params(axis='x', labelsize=8)
axs[1].tick_params(axis='x', labelsize=8)
axs[2].tick_params(axis='x', labelsize=8)
blackPal=sns.color_palette('blend:0.1,blue,0.9', n_colors=10)[0]
palette=sns.color_palette("Blues",n_colors=7)
palette[0]=blackPal
sns.histplot(data=pdTot[(pdTot[["$Z_A$"]] != 0).all(axis=1)], x="$Z_A$", hue="Converged in Batch",multiple="stack",discrete = True, binwidth=1, ax=axs[0],legend=False,palette=palette)
sns.histplot(data=pdTot[(pdTot[["$Z_B$"]] != 0).all(axis=1)], x="$Z_B$", hue="Converged in Batch",multiple="stack",discrete = True,binwidth=1, ax=axs[1],legend=False,palette=palette)
sns.histplot(data=pdTot, x="$Z_C$", hue="Converged in Batch",discrete = True,multiple="stack", binwidth=1, ax=axs[ 2],legend=False,palette=palette)
DNC = mpatches.Patch(color=palette[0]
, label='DNR')
a = mpatches.Patch(color=palette[1]
, label='0')
b = mpatches.Patch(color=palette[2]
, label='1')
c = mpatches.Patch(color=palette[3]
, label='2')
d = mpatches.Patch(color=palette[4]
, label='3')
e = mpatches.Patch(color=palette[5]
, label='4')
f = mpatches.Patch(color=palette[6]
, label='5')
s = fig.subplotpars
bb=[s.left+0.03, s.top-0.05, s.right-s.left, 0.05 ]
fig.legend([DNC,a,b,c,d,e,f],  ["DNR",0,1,2,3,4,5],title="$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$", loc=8, bbox_to_anchor=bb, ncol= 7, borderaxespad=0,
                bbox_transform=fig.transFigure, fancybox=False, edgecolor="k")

plt.tight_layout(rect=[0, 0.03, 1, .87])
plt.savefig('distPolishedtunedMagn.png',dpi=1200,bbox_inches='tight')
plt.clf()


#Substrate distribution of converged films
fig,(ax,ax2) = plt.subplots(1,2, sharey=True, facecolor='w', gridspec_kw={'width_ratios': [1, 1]},figsize=(5, 3))
sns.histplot(data=pdTot, x="$Z_{Subst}$", hue="Converged in Batch",multiple="stack",discrete = True, binwidth=1, ax=ax,legend=False,palette=palette)

sns.histplot(data=pdTot, x="$Z_{Subst}$", hue="Converged in Batch",multiple="stack",discrete = True, binwidth=1, ax=ax2,legend=False,palette=palette)
ax.tick_params(axis='x', labelsize=8)
ax2.tick_params(axis='x', labelsize=8)
ax.set_xticks(np.arange(45,48,1))
ax2.set_xticks(np.arange(77,80,1))
ax.set_xlim(44, 48)
ax2.set_xlim(76, 80)
ax.set_xlabel("")
ax2.set_xlabel("")
ax.spines['right'].set_visible(False)
ax2.spines['left'].set_visible(False)
fig.text(0.56, 0.00001, 'Substrate Elements', ha='center')
ax.yaxis.tick_left()
ax2.yaxis.tick_right()
ax.set_xticks(ticks=[45,46,47],labels=["Rh","Pd","Ag"])
ax2.set_xticks(ticks=[77,78,79],labels=["Ir","Pt","Au"])
d = .015
kwargs = dict(transform=ax.transAxes, color='k', clip_on=False)
ax.plot((1-d,1+d), (-d,+d), **kwargs)
ax.plot((1-d,1+d),(1-d,1+d), **kwargs)
kwargs.update(transform=ax2.transAxes)
ax2.plot((-d,+d), (1-d,1+d), **kwargs)
ax2.plot((-d,+d), (-d,+d), **kwargs)
plt.tight_layout()
plt.savefig("distPolishedTunedSubst.png",dpi=1200,bbox_inches='tight')
plt.clf()

#Visualization of the films convergence rate having 2 and 1 magnetic layers 
pa=palette
palette=[palette[6],palette[5],palette[4],palette[3],palette[2],palette[1],palette[0]]
labels = ["0","1","2","3","4","5","DNR"]
sizes = pdTot.query("`$Z_A$`==0 & `$Z_B$`>0").groupby("Converged in Batch").agg("count")["$Z_B$"]
labels_inner = ["0","1","2","3","4","5","DNR"]
sizes_inner = pdTot.query("`$Z_B$`==0").groupby("Converged in Batch").agg("count")["$Z_B$"]
colors = palette
colors_inner= palette
plt.pie(sizes,  colors=colors, startangle=90,frame=True)
plt.pie(sizes_inner,colors=colors_inner,radius=0.75,startangle=90)
centre_circle = plt.Circle((0,0),0.5,color='black', fc='white',linewidth=0)
fig = plt.gcf()
fig.gca().add_artist(centre_circle)
plt.axis('equal')
DNC = mpatches.Patch(color=pa[0]
, label='DNR')
a = mpatches.Patch(color=pa[1]
, label='5')
b = mpatches.Patch(color=pa[2]
, label='4')
c = mpatches.Patch(color=pa[3]
, label='3')
d = mpatches.Patch(color=pa[4]
, label='2')
e = mpatches.Patch(color=pa[5]
, label='1')
f = mpatches.Patch(color=pa[6]
, label='0')
art=[f,e,d,c,b,a,DNC]
plt.legend( handles=art, fancybox=False, edgecolor="k",title="$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.tight_layout()
plt.savefig("emptyAB.png",dpi=1200)
plt.clf()

#Convergence per batch
sns.set_theme(style="whitegrid")
fig, axs = plt.subplots(2, 2, figsize=(7, 7))
sns.histplot(data=df[(df[["$Z_A$"]] != 0).all(axis=1)], x="$Z_A$", hue="Converged in Batch",hue_order = pd.unique(df['Converged in Batch'])[::-1], multiple="stack",discrete = True, binwidth=1, ax=axs[0, 0])
sns.histplot(data=df[(df[["$Z_B$"]] != 0).all(axis=1)], x="$Z_B$", hue="Converged in Batch",hue_order = pd.unique(df['Converged in Batch'])[::-1], multiple="stack",discrete = True,binwidth=1, ax=axs[0, 1])
sns.histplot(data=df, x="$Z_C$", hue="Converged in Batch",hue_order = pd.unique(df['Converged in Batch'])[::-1], multiple="stack",discrete = True, binwidth=1, ax=axs[1, 0])
sns.histplot(data=df, x="$Z_{Subst}$", hue="Converged in Batch",hue_order = pd.unique(df['Converged in Batch'])[::-1], multiple="stack",discrete = True, binwidth=1, ax=axs[1, 1])
plt.tight_layout()
plt.savefig('batchDistPolished.png',dpi=600)
plt.clf()

#Some more plots showing the distributions across DFT IntMl iterations
sns.set_theme(style="whitegrid")
sns.histplot(data=df, x='Converged in Batch',discrete = True, binwidth=1)
plt.tight_layout()
plt.xlabel('Converged in Batch')
plt.savefig('batchDistPolishedCountsPerBatch.png',dpi=600)
plt.clf()

sns.set_theme(style="whitegrid")
sns.relplot(data=df[["$n_{Iter}^{Force}$","Converged in Batch"]],y="$n_{Iter}^{Force}$",x="Converged in Batch")
plt.tight_layout()
plt.savefig('batchtoPolished.png',dpi=600)
plt.clf()

#Number of force iterations required to reach a threshold of 10^-3 as force criterion across batches as boxplot distribution
sns.set_theme(style="whitegrid")
ax=sns.boxplot(data=df[["Force Iterations to $10^{-3}$","Converged in Batch"]],y="Force Iterations to $10^{-3}$",x="Converged in Batch",showmeans=True,meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},medianprops=dict(color="red", alpha=0.7),whiskerprops = dict(linestyle='-',linewidth=.5, color='grey'),
            flierprops=dict(markerfacecolor="#707070", marker="d"),palette=reversed(sns.color_palette("Blues")))
medians = df.groupby(["Converged in Batch"])["Force Iterations to $10^{-3}$"].quantile(q=0.25).values
nobs = df["Converged in Batch"].value_counts().sort_index().values
nobs = [str(x) for x in nobs.tolist()]
ax.xaxis.set_major_formatter(FuncFormatter(lambda x, _: int(x)))
plt.tight_layout()
plt.xlabel("$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.ylabel("$n^{10^{-3}}_{\\mathrm{Force\ Iter}}$")
plt.savefig('batchto10-3PolishedBox.png',dpi=1200)
plt.clf()

#Needed SCF iterations to reach convergence across batches as boxplot distribution
sns.set_theme(style="whitegrid")
ax=sns.boxplot(data=df[["$n_{Iter}^{Tot}$","Converged in Batch"]],y="$n_{Iter}^{Tot}$",x="Converged in Batch",palette=reversed(sns.color_palette("Blues")),meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},showmeans=True,flierprops=dict(markerfacecolor="#707070", marker="d"),medianprops=dict(color="red", alpha=0.7),whiskerprops = dict(linestyle='-',linewidth=.5, color='grey'))
ax.xaxis.set_major_formatter(FuncFormatter(lambda x, _: int(x)))
plt.xlabel("$n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.ylabel("$n_{\\mathrm{Iter}}^{\\mathrm{Tot\ SCF}}$")
plt.tight_layout()
plt.savefig('batchtoTotItBoxPolished.png',dpi=1200)
plt.clf()

#Compute averages for SCF and force iterations
meta=df.groupby('Converged in Batch').agg(avg_Forceit=('Force Iterations to $10^{-3}$','mean'),
                       mean_it=('Force Iterations to $10^{-3}$','mean'),meanForceiter=("$n_{Iter}^{Force}$","mean"),
                       count=('Force Iterations to $10^{-3}$','count'),mean_totIt=("$n_{Iter}^{Tot}$","median"),avg_totIt=("$n_{Iter}^{Tot}$","mean"),).reset_index()

#Lineplot showing the total required number of force and SCF iterations per each batch
sns.set_theme(style="darkgrid")
mean = df.groupby('Converged in Batch')["$n_{Iter}^{Tot}$"].mean()
d=1-0.682
p025 = df.groupby('Converged in Batch')["$n_{Iter}^{Tot}$"].quantile(d/2)
p975 = df.groupby('Converged in Batch')["$n_{Iter}^{Tot}$"].quantile(1-d/2)
ax=sns.lineplot(data=meta, x="Converged in Batch", y="avg_totIt",legend=False)
plt.ylabel("Average $n_{\\mathrm{Iter}}^{\\mathrm{Tot\ SCF}}$")
ax2=ax.twinx()
sns.lineplot(data=meta, x="Converged in Batch", y="meanForceiter",color="r",legend=False,ax=ax2)
ax.set_xlabel("Average $\\displaystyle n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
mean = df.groupby('Converged in Batch')['$n_{Iter}^{Force}$'].mean()
d=1-0.682
p025 = df.groupby('Converged in Batch')['$n_{Iter}^{Force}$'].quantile(d/2)
p975 = df.groupby('Converged in Batch')['$n_{Iter}^{Force}$'].quantile(1-d/2)
ax2.spines['right'].set_color('r')
ax2.set_ylabel("Average $\\displaystyle n^{\\mathrm{Tot\ Conv}}_{\\mathrm{Force\ Iter}}$")
ax2.tick_params(axis='y', colors='r')
ax2.set_yticks(np.linspace(ax2.get_yticks()[0], ax2.get_yticks()[-1], len(ax.get_yticks())))
ax2.set_yticks([7.5,8,8.5,9,9.5,10,10.5])
ax2.grid(None)
ax.xaxis.set_major_formatter(FuncFormatter(lambda x, _: int(x)))
plt.tight_layout()
plt.savefig("improvA.png",dpi=1200)
plt.clf()

#Lineplot showing the required number of force iterations needed to reach 10^-3 as force threshold per each batch
sns.set_theme(style="darkgrid")
ax=sns.lineplot(data=meta, x="Converged in Batch", y="avg_Forceit")
plt.ylabel("Average $\\displaystyle n^{10^{-3}}_{\\mathrm{Force\ Iter}}$")
plt.tight_layout()
ax.xaxis.set_major_formatter(FuncFormatter(lambda x, _: int(x)))
ax.set_xlabel("$\\displaystyle n_{\\mathrm{Iter}}^{\\mathrm{DFT\ IntML}}$")
plt.savefig("improvB.png",dpi=1200)
plt.clf()

#Boxplot showing the distribution of required number of force iterations needed to reach 10^-5 as force threshold per each batch
sns.set_theme(style="whitegrid")
ax=sns.boxplot(data=df[["$n_{Iter}^{Tot}$","Converged in Batch"]],y="$n_{Iter}^{Tot}$",x="Converged in Batch",showmeans=True,meanprops={"marker":"o",
                       "markerfacecolor":"white", 
                       "markeredgecolor":"black",
                      "markersize":"6"},medianprops=dict(color="red", alpha=0.7),whiskerprops = dict(linestyle='-',linewidth=.5, color='grey'),
            flierprops=dict(markerfacecolor="#707070", marker="d"))
medians = df.groupby(["Converged in Batch"])["$n_{Iter}^{Tot}$"].quantile(q=0.25).values
nobs = df["Converged in Batch"].value_counts().values
nobs = [str(x) for x in nobs.tolist()]
nobs = ["n: " + i for i in nobs]
pos = range(len(nobs))
for tick,label in zip(pos,ax.get_xticklabels()):
    ax.text(pos[tick],
            medians[tick] - 1.5,
            nobs[tick],
            horizontalalignment='center',
            size='small',
            color='black',
            weight='bold')
ax.xaxis.set_major_formatter(FuncFormatter(lambda x, _: int(x)))
plt.tight_layout()
plt.savefig('batchto10-5TotItPolishedBox.png',dpi=600)
plt.clf()



